require 'socket'
require 'io/wait'
require 'io/splice'
require 'io/nonblock'
require "test/unit"

class TestTCPCopyStream < Test::Unit::TestCase
  def setup
    host = ENV["TEST_HOST"] || "127.0.0.1"
    @srv = TCPServer.new(host, 0)
    @port = @srv.addr[1]
    @client = TCPSocket.new(host, @port)
    @client.setsockopt Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1
    @accept = @srv.accept
    @accept.setsockopt Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1
    @client.sync = @accept.sync = true
    @r, @w = IO.pipe
  end

  def teardown
    @srv.close
    [ @client, @accept, @r, @w ].each { |io| io.close unless io.closed? }
  end

  def test_client_to_server_eof
    nr = 2000
    buf = '0123456789abcdef' * 1024
    expect = buf.size * nr
    thr = Thread.new do
      nr.times { @client.write(buf) }
      @client.close
    end
    sleep 1 # wait for rcvbuf to fill up
    bytes = IO::Splice.copy_stream(@accept, "/dev/null")
    assert_equal expect, bytes
  end

  def test_client_to_server_expect
    nr = 2000
    buf = '0123456789abcdef' * 1024
    expect = buf.size * nr
    thr = Thread.new do
      nr.times { @client.write(buf) }
    end
    sleep 1 # wait for rcvbuf to fill up
    bytes = IO::Splice.copy_stream(@accept, "/dev/null", expect)
    assert_equal expect, bytes
  end
end
